{% load django_listing %}
{% if listing.has_group_by == True %}
    {% get_uniq_id as uniq_id %}
    <div class="btn-group-sm me-2 toolbar_item {{ item.css_class }}" role="group">
        <a id="toolbar-item-{{ uniq_id }}-update-id"
           class="button-action-group-by {{ item.theme_button_class }}">
            {{ item.label }}
        </a>
    </div>
{% endif %}